@@ -24,7 +24,7 @@
 
 #include <config.h>
 
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
 
 #include "nm-vpn-password-dialog.h"
