@@ -40,7 +40,7 @@ mobile_helper_get_status_pixbuf (guint32 quality,
 {
 	GdkPixbuf *pixbuf, *qual_pixbuf, *wwan_pixbuf, *tmp;
 
-	wwan_pixbuf = nma_icon_check_and_load ("nm-wwan-tower", &applet->wwan_tower_icon, applet);
+	wwan_pixbuf = nma_icon_check_and_load ("nm-wwan-tower", applet);
 
 	if (!quality_valid)
 		quality = 0;
@@ -71,7 +71,7 @@ mobile_helper_get_status_pixbuf (guint32 quality,
 
 	/* And finally the roaming or technology icon */
 	if (state == MB_STATE_ROAMING) {
-		tmp = nma_icon_check_and_load ("nm-mb-roam", &applet->mb_roaming_icon, applet);
+		tmp = nma_icon_check_and_load ("nm-mb-roam", applet);
 		gdk_pixbuf_composite (tmp, pixbuf, 0, 0,
 		                      gdk_pixbuf_get_width (tmp),
 							  gdk_pixbuf_get_height (tmp),
@@ -96,15 +96,15 @@ GdkPixbuf *
 mobile_helper_get_quality_icon (guint32 quality, NMApplet *applet)
 {
 	if (quality > 80)
-		return nma_icon_check_and_load ("nm-signal-100", &applet->wifi_100_icon, applet);
+		return nma_icon_check_and_load ("nm-signal-100", applet);
 	else if (quality > 55)
-		return nma_icon_check_and_load ("nm-signal-75", &applet->wifi_75_icon, applet);
+		return nma_icon_check_and_load ("nm-signal-75", applet);
 	else if (quality > 30)
-		return nma_icon_check_and_load ("nm-signal-50", &applet->wifi_50_icon, applet);
+		return nma_icon_check_and_load ("nm-signal-50", applet);
 	else if (quality > 5)
-		return nma_icon_check_and_load ("nm-signal-25", &applet->wifi_25_icon, applet);
+		return nma_icon_check_and_load ("nm-signal-25", applet);
 
-	return nma_icon_check_and_load ("nm-signal-00", &applet->wifi_00_icon, applet);
+	return nma_icon_check_and_load ("nm-signal-00", applet);
 }
 
 GdkPixbuf *
@@ -112,23 +112,23 @@ mobile_helper_get_tech_icon (guint32 tech, NMApplet *applet)
 {
 	switch (tech) {
 	case MB_TECH_1XRTT:
-		return nma_icon_check_and_load ("nm-tech-cdma-1x", &applet->mb_tech_1x_icon, applet);
+		return nma_icon_check_and_load ("nm-tech-cdma-1x", applet);
 	case MB_TECH_EVDO:
-		return nma_icon_check_and_load ("nm-tech-evdo", &applet->mb_tech_evdo_icon, applet);
+		return nma_icon_check_and_load ("nm-tech-evdo", applet);
 	case MB_TECH_GSM:
 	case MB_TECH_GPRS:
-		return nma_icon_check_and_load ("nm-tech-gprs", &applet->mb_tech_gprs_icon, applet);
+		return nma_icon_check_and_load ("nm-tech-gprs", applet);
 	case MB_TECH_EDGE:
-		return nma_icon_check_and_load ("nm-tech-edge", &applet->mb_tech_edge_icon, applet);
+		return nma_icon_check_and_load ("nm-tech-edge", applet);
 	case MB_TECH_UMTS:
-		return nma_icon_check_and_load ("nm-tech-umts", &applet->mb_tech_umts_icon, applet);
+		return nma_icon_check_and_load ("nm-tech-umts", applet);
 	case MB_TECH_HSDPA:
 	case MB_TECH_HSUPA:
 	case MB_TECH_HSPA:
 	case MB_TECH_HSPA_PLUS:
-		return nma_icon_check_and_load ("nm-tech-hspa", &applet->mb_tech_hspa_icon, applet);
+		return nma_icon_check_and_load ("nm-tech-hspa", applet);
 	case MB_TECH_LTE:
-		return nma_icon_check_and_load ("nm-tech-lte", &applet->mb_tech_lte_icon, applet);
+		return nma_icon_check_and_load ("nm-tech-lte", applet);
 	case MB_TECH_WIMAX:
 	default:
 		return NULL;
